///////////////////////////////////////////////////////////////////////////////
//  programma AvviaBackup
//
//  Piero Pratesi, aprile 2013
//  Programma per XP 32/64 bit, compilabile con lazarus/free pascal.
//  Progettato per l'uso in abbinamento a bart.exe v3.00 del 23/12/2012 .
//
//  SCOPO:
//  Elimina il problema della variazione del nome del drive su cui viene
//  effettuato il backup, usando uno dei parametri di bart.exe.
//  Prevede che bart.exe sia installato nella stessa cartella di questo programma.
//  Nella radice del disco che riceverà il backup deve esistere una sottocartella
//  di specifico nome, ad es. "ZZZZZZZZZZ_NON_CAMBIARE_NOME", contenente un file
//  di specifico nome, ad es. "ZZZZZZZZZZ_SEGNAPOSTO_BACKUP.TXT" (il cui
//  contenuto è ininfluente).
//
//  COME:
//  Legge il nome del disco nel quale esistono cartella e file segnaposto e
//  confeziona una riga di comando per bart con i parametri passati a questo
//  programma senza apportarvi modifiche. Se uno di tali parametri è '-letter:'
//  ci appende il nome del disco su cui è situato il segnaposto.
//
//  LICENZA:
//  This software is released under the "MIT License"
//  (http://en.wikipedia.org/wiki/MIT_License):
//
//  Copyright (C) 2013 Piero Pratesi, Rome, Italy
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to
//  deal in the Software without restriction, including without limitation the
//  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
//  sell copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
//  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
//  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
//  USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//  MEMO
//  bart.exe v3.00, 23 dicembre 2012, (http://www.zhornsoftware.co.uk)
//  Command line for part.exe
//  The following can be specified on the command line:
//  -runall - Bart will run all enabled jobs, show the balloon summary and quit
//  -show - When used with -runall, the main dialog is displayed while the jobs
//   are running
//  -stay - Bart will stick around once it's finished running the jobs.
//   Clicking any buttons in the interface if -show is specified will also
//   cause it to remain
//  -shutdown - the computer will be shut down once the jobs have been run, as
//   long as there were no errors in any of the tasks
//  -ignoreerrors - if -shutdown has been specified and errors were found, shut
//   down anyway
//  -datelog - save detailed logs to different files, named as the date on which
//   the job was run
//  -logexclusions - also write files which are skipped to the detailed log file
//  -file <filename> - open the specified job file
//  -noballoon - do not display a tray icon balloon when the job has completed
//  -auto - run silently with no user interaction, or windows: use this for
//   running Bart from a Scheduled Task
//  -showdetlog - open the detailed log file when the job completes
//  -letter:x - set 'x' as the variable drive
///////////////////////////////////////////////////////////////////////////////

program AvviaBackup;

{$ifdef fpc}{$mode delphi}{$endif}
{$apptype console}

uses
  Windows,
  Classes,
  SysUtils,
  Process,
  crt;

const
  SegnapostoBackup = 'ZZZZZZZZZZ_NON_CAMBIARE_NOME\ZZZZZZZZZZ_SEGNAPOSTO_BACKUP.TXT';
  NomeBart = 'bart.exe';

{$R *.res}

  // MAIN
var
  Drive: char;
  DriveLetter: string;
  DriveAdatto: boolean;
  AProcess: TProcess;
  ErroreAvvio: boolean;
  ErroreMancaParametro: boolean;
  LineaComando: string;
  i: integer;

begin

  WriteLn('PP - aprile 2013');
  WriteLn('Ricerca ed avvio del programma '+ NomeBart + ' per il backup');
  WriteLn();

  // Cercali in ordine inverso, è probabile che le chiavette USB
  // siano elencate per ultime
  // I drive sono adatti solo se:
  // REMOVABLE (li chiama floppy, ma sono anche le chiavette USB)
  // REMOTE (utile se si fa un backup in rete)
  for Drive := 'Z' downto 'C' do
  begin
    DriveLetter := Drive + ':\';
    DriveAdatto := False;

    case GetDriveType(PChar(DriveLetter)) of
      DRIVE_REMOVABLE, DRIVE_REMOTE:
      begin
        DriveAdatto := True;
      end;
      // per questi drive nessuna azione, si può usare bart.exe da solo
      // DRIVE_UNKNOWN
      // DRIVE_NO_ROOT_DIR
      // DRIVE_FIXED
      // DRIVE_CDROM
      // DRIVE_RAMDISK
    end;

    if (DriveAdatto) then
    begin

      if FileExists(DriveLetter + SegnapostoBackup) then
      begin // trovato segnaposto, avvia bart.exe
        AProcess := TProcess.Create(nil);
        AProcess.Executable := NomeBart;

        LineaComando := '';
        ErroreMancaParametro := True;
        i := 1;
        while ParamStr(i) <> '' do
        begin
          LineaComando := LineaComando + ' ' + ParamStr(i);
          if ParamStr(i) = '-letter:' then
          begin
            ErroreMancaParametro := False;
            LineaComando := LineaComando + Drive;
            AProcess.Parameters.Add(ParamStr(i) + Drive);
          end
          else
          begin
            AProcess.Parameters.Add(ParamStr(i));
          end;
          Inc(i);
        end; // while
        writeln('Parametri di avvio di bart.exe:' + LineaComando);

        // dopo l'avvio del backup, continua la tua esecuzione
        AProcess.Options := AProcess.Options + [poDefaultErrorMode];
        // avvia l'esecuzione di bart.exe
        ErroreAvvio := False;
        try
          if not ErroreMancaParametro then
            AProcess.Execute;
        except
          ErroreAvvio := True;
        end;
        AProcess.Free;
        // ed esci dal for
        break;
      end; // if

    end; // if

  end; // for

  // se non trovato segnaposto o accaduti errori fermati e mostra avviso
  if (not DriveAdatto) or ErroreAvvio or ErroreMancaParametro then
  begin
    Write('ERRORE: ');
    if not DriveAdatto then
    begin
      writeln('in nessun drive rimovibile o di rete esiste il segnaposto:');
      writeln(SegnapostoBackup);
    end;
    if ErroreMancaParametro then
    begin
      writeln('parametro "-letter:" non specificato, inutile usare questo programma.');
    end;
    if ErroreAvvio then
    begin
      writeln('durante l''avvio del programma ' + NomeBart);
    end;

    // se problemi, fermati per mostrarli
    writeln();
    WriteLn('Premi <ENTER> per terminare questo programma.');
    ReadLn(DriveLetter);
  end
  else // nessun errore
  begin
    WriteLn(NomeBart + ' correttamente avviato.');
    delay(5000); // delay to autoclose dos window, any value >= 1
  end;

end.
